<?xml version="1.0" encoding="ISO-8859-1" ?>
<!--
# $Id: todo_tasks.xsl,v 1.4 2003/08/22 13:19:15 camille Exp $

# Copyright 2001 MandrakeSoft S.A., tous droits rservs

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Project         : Mandrake Linux
# Module          : Borges
# File            : todo_tasks.xsl
# Author          : Camille Bgnis
# Created On      : Mon Jul 28 16:25:21 2003
# Copyright       : Camille Bgnis camille@mandrakesoft.com,
#                    MandrakeSoft under the GPL license
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

This stylesheets takes all revision histories
(reports/revhistories.xml) as input and outputs an HTML page listing
all authors with pending tasks. Additionaly, it generates one HTML
page per contributor, with a summary table of all tasks waiting for
completion by contributors for all declared documents for one specific
major release. This page is the entry point for the web frontend.

The mail message for each author is also generated in parallel

-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.1"> 

<xsl:param name="conf-file" select="'../conf/repository.xml'"/>
<xsl:param name="release" select="'1.0'"/>
<xsl:param name="url" select="'modules/'"/>
<xsl:param name="output" select="'frontend/'"/>
<xsl:param name="cgi" select="'cgi-bin/web-frontend.cgi'"/>

<!-- This hash contains all todo tasks for all releases -->
<!--
translation
(not(../revision/@role='2translate') and (../../revhistory/revision[not(string(date)='YYYY-MM-DD')][@role='2translate'] ) ) 

original or original unknown
     (../revision/@role='2translate' and ((not(@role='1time') or @role='1time' and ( not(string(authorinitials)='tbn') or not(../../revhistory/revision[not(string(authorinitials)='tbn')]) ) )))

todo?
not(following-sibling::revision) or following-sibling::revision[1][not(string(date)='YYYY-MM-DD')]

-->


<xsl:output method="xml" encoding="UTF-8" indent="yes"/>

<xsl:key name="tasks" match="revision[string(date)='YYYY-MM-DD'][(not(following-sibling::revision) or following-sibling::revision[1][not(string(date)='YYYY-MM-DD')]) and ((../revision/@role='2translate' and ((not(@role='1time') or @role='1time' and not(string(authorinitials)='tbn') or not(../../revhistory/revision[not(string(authorinitials)='tbn')]) ))) or (not(../revision/@role='2translate') and (../../revhistory//revision[not(string(date)='YYYY-MM-DD')][@role='2translate'] ) )) ]" use="authorinitials"/>


<xsl:template match="/">



  <xsl:variable name="title">
    <xsl:value-of select="document($conf-file,.)//title[1]"/>
    Release <xsl:value-of select="$release"/>
  </xsl:variable>

  <html>
    <head>
      <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8"/>
      <META HTTP-EQUIV="Expires" CONTENT="Mon, 18 May 1974 18:29:02 GMT"/>
      <title><xsl:value-of select="$title"/></title>
    </head>
    <body bgcolor="white" text="black" link="red" vlink="brown" alink="black">
      <h1><xsl:value-of select="$title"/></h1>


      <p>
        <a>
          <xsl:attribute name="href">/~<xsl:value-of select="$cgi"/>?action=regen</xsl:attribute>
          Regenerate Frontend
        </a>
      </p>
      <table border="1">
        <tr>
          <th>Author</th>
          <th>Tasks</th>
        </tr>

<!-- for each author having todo tasks -->        
        <xsl:for-each select="//revision[generate-id(.)=generate-id(key('tasks',authorinitials))]">
          <xsl:sort select="authorinitials"/>

          <xsl:variable name="author">
            <xsl:value-of select="authorinitials"/>                  
          </xsl:variable>
            
          <tr role="author">
            <td role="author">
              <a>
                <xsl:attribute name="href"><xsl:value-of select="authorinitials"/>.html</xsl:attribute>
                <xsl:value-of select="authorinitials"/>
              </a>
            </td>
            <td role="tasks"><xsl:value-of select="count(key('tasks',authorinitials))"/></td>
          </tr>
            
          <!-- Output author HTML frontend page -->
              <xsl:document method="xml" href="{$author}.html">
                
                <html>
                  <head>
                    <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8"/>
                    <META HTTP-EQUIV="Expires" CONTENT="Mon, 18 May 1974 18:29:02 GMT"/>
                    <title><xsl:value-of select="$author"/> Tasks List</title>
                  </head>
                  <body bgcolor="white" text="black" link="red" vlink="brown" alink="black">
                    <h1><xsl:value-of select="$author"/> Tasks List</h1>
                    
                    <table border="1">
                      <tr>
                        <th>Task</th>
                        <th>Lang</th>
                        <th>Download</th>
                        <th>Edit</th>
                        <th>Upload</th>
                        <xsl:if test="$author='tbn'">
                          <th>Author</th>
                        </xsl:if>
                      </tr>
                      
                      <xsl:for-each select="key('tasks',authorinitials)">
                        <xsl:sort select="revremark"/>
              
                        <xsl:variable name="revremark" select="revremark"/>
                        <xsl:variable name="language" select="../@lang"/>
                        <xsl:variable name="module" select="../../@id"/>

                        <!--

                        <xsl:if test="$module='audio-apps-section' and $language='en'">
                <xsl:message>
                <xsl:value-of select="$language"/>     <xsl:text> </xsl:text>
                <xsl:copy-of select="../.."/>   
                <xsl:value-of select="$module"/>     <xsl:text>
              </xsl:text>
              <xsl:value-of select="not(../../revhistory/revision[not(string(date)='YYYY-MM-DD')][@role='2translate'] )"/>     <xsl:text>
            </xsl:text>
          </xsl:message>
        </xsl:if>
        -->
                        <tr>
                          <th><xsl:value-of select="$revremark"/></th>
                          <td><xsl:value-of select="$language"/></td>
                          <td>
                            <a>
                              <xsl:attribute name="href">/~<xsl:value-of select="$url"/>/<xsl:value-of select="$language"/>/<xsl:value-of select="$module"/>.xml
                              </xsl:attribute>
                              <xsl:value-of select="$module"/>.xml
                            </a>
                          </td>
                          <td>
                            <a>
                              <xsl:attribute name="href">/~<xsl:value-of select="$cgi"/>?action=edit&amp;task=<xsl:value-of select="$revremark"/>&amp;module=<xsl:value-of select="$module"/>&amp;language=<xsl:value-of select="$language"/>&amp;author=<xsl:value-of select="$author"/></xsl:attribute>
                              Edit
                            </a>
                          </td>
                          <td>
                            <FORM action="/~{$cgi}" enctype="multipart/form-data" method="post">
                              <INPUT type="file" name="module-file"/>
                              <input type="hidden" name="action" value="check"/>
                              <input type="hidden" name="author" value="{$author}"/>
                              <input type="hidden" name="module" value="{$module}"/>
                              <input type="hidden" name="language" value="{$language}"/>
                              <input type="hidden" name="task" value="{$revremark}"/>
                              <INPUT type="submit" value="Send"/>
                            </FORM>
                          </td>
                          <xsl:if test="$author='tbn'">
                            <td>
                              <FORM action="/~{$cgi}" enctype="multipart/form-data" method="post">
                                <INPUT type="submit" value="Attribute task to:"/>
                                <INPUT type="input" name="new-author"/>
                                <input type="hidden" name="action" value="author"/>
                                <input type="hidden" name="author" value="{$author}"/>
                                <input type="hidden" name="module" value="{$module}"/>
                                <input type="hidden" name="language" value="{$language}"/>
                                <input type="hidden" name="task" value="{$revremark}"/>
                              </FORM>
                            </td>
                          </xsl:if>
                        </tr>
                        
                      </xsl:for-each>
                      
                      
                    </table>
                  </body>
                </html>
            
            
              </xsl:document>
            
              <!-- Output author tasks email message -->
              <xsl:if test="not($author='tbn')">
                <xsl:document method="text" href="{$author}.mail">
                
                  <xsl:for-each select="key('tasks',authorinitials)">
                    <xsl:sort select="revremark"/>
                    
                    <xsl:variable name="revremark" select="revremark"/>
                    <xsl:variable name="language" select="../@lang"/>
                    <xsl:variable name="module" select="../../@id"/>
                    <xsl:text></xsl:text>
<xsl:value-of select="$revremark"/> of <xsl:value-of select="$module"/> in "<xsl:value-of select="$language"/>";
<xsl:text></xsl:text>
                    
                  </xsl:for-each>
                  
                </xsl:document>
              </xsl:if>
            
            </xsl:for-each>
      
          </table>
        </body>
      </html>
  
</xsl:template>


</xsl:stylesheet>
