#!/usr/bin/perl -w
#---------------------------------------------------------------
# Project         : Linux-Mandrake
# Module          : bin
# File            : web-frontend.cgi
# Author          : Camille Bgnis
# Created On      : 2003-07-28
# Copyright       : Camille Bgnis <camille@mandrakesoft.com>,
#                    MandrakeSoft under the GPL license
#---------------------------------------------------------------
# 
# This CGI script is responsible for displaying web frontend pages

use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser);

# Security ENV
$ENV{'PATH'} = '/bin:/usr/bin';

# frontend relative URL
# meant to be updated at publication time
my $url="borges-frontend/";

# Borges repository path in local filesystem
# meant to be updated at publication time
my $localpath="/repository";

# Available module outputs formats
# meant to be updated at publication time
my $formats="pdf html";
my @formats=split(' ',$formats);

# Initiate CGI
my $query = new CGI;

# Set to 1 for debugging messages 
my $debug=0;

# Common CGI params
my $module;
my $language;
my $author;
my $task;
my $type;

&check_params;
	
if ( my $action=$query->param('action') ) {
    if ( $action eq 'edit' ) {

	print $query->header(-expires=>'now'),
	$query->start_html("Editing $module in $language"),
	$query->h1("Editing <i>$module</i> in <i>$language</i> for task <i>$task</i>");
	

	open(FILE, "< $localpath/modules/$language/$module.xml") || die "Open error: \"$localpath/modules/$language/$module.xml\" $!\n";
	flock(FILE, 2) or die "cannot lock file: $!";
	my $content="";
	while(<FILE>) {
	    $content.=$_;
	}
        close FILE;
	
	&edit($content);

    } elsif ( $action eq 'check' ) {

	my $moduletext="";

# If file comes from upload read it
	if ( $query->param('module-file') ) {
	    my $uploaded=$query->param('module-file');
	    while(<$uploaded>) {
		$moduletext.=$_;
	    }
	    close FILE;
	    $query->param('moduletext',$moduletext);
	} else {
	    $moduletext=$query->param('moduletext');
	}
# fix DOS like newline... (why?)
	$moduletext=~s/\r//g;

	my $tempfile="$localpath/modules/$language/tmp-$module.xml";

	open(FILE, "> $tempfile") || die "Open error: \"$localpath/modules/$language/tmp-$module.xml\" $!\n";
	flock(FILE, 2) or die "cannot lock file: $!";
	print FILE $moduletext;
        close FILE;

	print $query->header(-expires=>'now'),
	$query->start_html("Checking $module in $language validity"),
	$query->h1("Checking <i>$module</i> in <i>$language</i> validity"),
	$query->p("Validating Module... Please wait...");

# Try validating module
	&execute("make -C $localpath/modules/$language/ tmp-$module.validate");
	
	open(FILE, "< $localpath/modules/$language/tmp-$module.validate") || die "Open error: \"$localpath/modules/$language/tmp-$module.validate\" $!\n";
	flock(FILE, 2) or die "cannot lock file: $!";
	my $content="";
	while(<FILE>) {
	    $content.=$_;
	}
        close FILE;

	chomp($content);

# If there are errors correct them
	if ($content) {
	    
	    print $query->h1("Errors Found!");
	    print $query->pre(&escape($content));
	    
	    print $query->p("You can correct errors below or upload again the file in your <a href=\"/$url/$author.html\">tasks list page</a>.");
	    &edit($moduletext);

# if no errors Commit file to CVS repository
	} else {

	
	    my $ID=$moduletext;
	    $ID=~s/.*<!-- \$Id[^,*]*,v ([\d.]*) .*//s;
	    $ID=$1;
	    
	    $query->h1("Committing <i>$module</i> in <i>$language</i>");
	    
	    if ( $ID eq "" ) {
		print $query->h1("No ID found in module, sorry, cannot commit....");
	    } else {
		print $query->p("Checking for possible conflicts... ");
# Check there are no conflicts
# and fix permissions
		&execute("cd $localpath/modules/$language/; cvs -z2 update -r $ID $module.xml; mv tmp-$module.xml $module.xml; chmod a+r $module.xml");
		my $result=&execute("cd $localpath/modules/$language/; cvs -z2 update -A $module.xml",1);
		
# If there are conflicts, edit them
		if ( $result=~/C $module/ ) {
		    
		    print $query->h1("Conflicts Found!");
		    
		    open(FILE, "< $localpath/modules/$language/$module.xml") || die "Open error: \"$localpath/modules/$language/$module.xml\" $!\n";
		    flock(FILE, 2) or die "cannot lock file: $!";
		    my $conflict_content="";
		    while(<FILE>) {
			$conflict_content.=$_;
		    }
		    close FILE;	    
# remove conflictuous file and get last version instead
		    &execute("cd $localpath/modules/$language/; rm $module.xml; cvs -z2 update -A $module.xml");
		    
		    print $query->p("Solve conflicts below:");
# Keep only last $Id field from conflicts
		    $conflict_content=~s/<<<<<<< $module.xml\s*<!-- \$I.: $module.xml,v $ID .*=======.*(<!-- \$I.: .* -->).*>>>>>>> [\d.]*/$1/s;
		    
		    &edit($conflict_content);

# If there are no conflicts, Commit to CVS and ask if we want to pass the task
		}else {
		    print $query->h2("Module looks sane :-)");
		    print $query->p("Committing module to repository... ");
		    &execute("make -C $localpath/modules/$language/ $module.commit CVSLOG=\"Automatic Web commit by $author\" ");
		    

		    print $query->p("Your module looks OK, you can now either:");
		    print "<ul>";
		    print " <li>Check your module output in";
		    foreach my $type (@formats) {
			&action_button('compile',$type);
		    }
		    print "</li>";
		    
		    print " <li>Declare the current task as finished:";
		    &action_button('revision',$task);
		    print "</li>";
		    
		    print "<li>Get back to editing your module by using your browser's 'back' button;";
		    print "<li>Get back to your <a href=\"/$url/$author.html\">tasks list</a>.";
		    print "</ul>";
	
		}
	    }
	}
	&execute("rm -f $localpath/modules/$language/tmp-$module.*");

# propose then various choices: see output, or get back to edition

	
    } elsif ( $action eq 'revision' )  {
	
	print $query->header(-expires=>'now'),
	$query->start_html("Changing Revision"),
	$query->h1("Passing revision <i>$task</i> for <i>$module</i> in <i>$language</i>");

	print $query->p("Processing Module... Please wait...");
	&execute("make -C $localpath/modules/$language/ $module.revision TYPE=$task AUTHOR=$author CVSLOG=\"Automatic Web commit by $author\"");

	print $query->p("Regenerating tasks list... Please wait...");
	&execute("make -C $localpath/reports publish-frontend");

	print $query->p("Tasks done, redirecting you to <a href=\"/$url/index.html\">Index page</a>.");

	print "<SCRIPT LANGUAGE=\"JavaScript\">
<!-- Begin
window.location=\"/$url/index.html\";
// End -->
</script>"

# Attribute 'tbn' task to a real author    
    } elsif ( $action eq 'author' )  {
	
	my $newauthor=$query->param('new-author');
	$newauthor=~s/(\w+)//;
	$newauthor=$1;

	print $query->header(-expires=>'now'),
	$query->start_html("Changing Author"),
	$query->h2("Changing Author for module <i>$module</i>, task <i>$task</i> in <i>$language</i> to <i>$newauthor</i>");

	print $query->p("Processing Module... Please wait...");
	&execute("make -C $localpath/modules/$language/ $module.revision TYPE=$task.todo AUTHOR=$newauthor CVSLOG=\"$author assigns $task to $newauthor\"");
	print $query->p("Regenerating tasks list... Please wait...");
	&execute("make -C $localpath/reports publish-frontend");

	print $query->p("Assignation succeeded, get back to <a href=\"/$url/index.html\">Index page</a> or <a href=\"/$url/$newauthor.html\">tasks page</a>.");

# regenerate Frontend pages    
    } elsif ( $action eq 'regen' )  {
	
	print $query->header(-expires=>'now'),
	$query->start_html("Regenerating Frontend"),
	$query->h2("Regenerating Frontend");

	print $query->p("Updating repository... Please wait...");
	&execute("make -C $localpath checkout");
	print $query->p("Regenerating tasks list... Please wait...");
	&execute("make -C $localpath/reports publish-frontend");

	print $query->p("Tasks done, redirecting you to <a href=\"/$url/index.html\">Index page</a>.");

	print "<SCRIPT LANGUAGE=\"JavaScript\">
<!-- Begin
window.location=\"/$url/index.html\";
// End -->
</script>"

# Compile module output    
    } elsif ( $action eq 'compile' )  {
	
	print $query->header(-expires=>'now'),
	$query->start_html("Compiling $module in $type"),
	$query->h2("Compiling $module in $type");

	print $query->p("Compilation in progress... Please wait...");
	&execute("make -C $localpath/manuals/module $module.$type");
# fix permissions so the browser can read resulting files
	&execute("chmod -R a+r $localpath/manuals/module/$module.$type");
# If the result is a directory, make it browsable too as well as other dirs in it
# In that case we must add a / at the end of URL (why?)
	if (-d "$localpath/manuals/module/$module.$type") {
	    &execute("find $localpath/manuals/module/$module.$type -type d -exec chmod a+x {} \\;");
	    $type.="/";
	}
	print $query->p("Compilation done, check it: <a href=\"/$url/repository/manuals/module/$module.$type\">$module.$type</a>.");

	print $query->p("Get back and commit your module by using your browser's 'back' button.");

    } else {
	print $query->header(-expires=>'now'),
	$query->h1("Action not implemented...");
    }
    
} else {
    print $query->header(-expires=>'now'),
    $query->h1("Error!");
}

print $query->end_html();

# Escape XML special chars
sub escape {
    my $text=$_[0];
    $text=~s/&/&amp;/gs;
    $text=~s/</&lt;/gs;
    return $text;
}

# Function responsible for running external commands 
# optional second argument is an acceptable exit error code
# (other than 0)
# other codes make script die
sub execute {
    my $command=$_[0];
    my $error_accept=0;
    if ( $_[1] ) { $error_accept=$_[1]};
    my $result=qx($command 2>&1);
    my $exit_value  = $? >> 8;

    if ( $debug == 1 ) {
	print $query->pre($command);
	print $query->pre($result);
	
    }

    if ( $exit_value != 0 && $exit_value != $error_accept ) {

	print $query->h2("Error while executing Command: $exit_value");
	print $query->pre($command);
	print $query->pre($result);
	
	print $query->h2("Send this page to Project administrator.");
	
	die;

    }

    return $result;
}

# Print editor textfield
sub edit {
    my $content=$_[0];
    print $query->start_form(-action=>"web-frontend.cgi"),
    $query->textarea(-name=>'moduletext',
		     -default=>$content,
		     -override=>1,
		     -rows=>30,
		     -columns=>90),$query->p;
    $query->param('action','check');
    &print_hidden;
    print $query->hidden('action'),
    $query->submit(-value=>'Send modified module'),
    $query->end_form;
}

# Print compilation button
sub action_button {
    my ($action,$label)=@_;
    print $query->start_form(-action=>"web-frontend.cgi");
    print $query->hidden('moduletext');
    $query->param('action',$action);
    print $query->hidden('action');
    &print_hidden;
    $query->param('type',$label);
    print $query->hidden('type'),
    $query->submit(-value=>$label),
    $query->end_form;
}

# Print hidden params for inter-scripts communications
sub print_hidden {
    print $query->hidden('module',$module),
    $query->hidden('language',$language),
    $query->hidden('author',$author),
    $query->hidden('task',$task);
}

# Get and Check tainted params
sub check_params {
    $module=$query->param('module');
    $language=$query->param('language');
    $author=$query->param('author');
    $task=$query->param('task');
    $type=$query->param('type');
#    my $metachars="&;`'\"|*?~<>^()[]{}$\n\r";

    $module=~s/([\w-]+)//;
    $module=$1;
    $language=~s/(\w\w)//;
    $language=$1;
    $author=~s/(\w+)//;
    $author=$1;
    $task=~s/(\w+)//;
    $task=$1;
    $type=~s/([\w\-.]+)//;
    $type=$1;
}
